package fitconnect.impl;

import com.nimbusds.jose.jwk.RSAKey;
import fitconnect.api.auth.OAuthService;
import fitconnect.api.auth.OAuthToken;
import fitconnect.api.crypto.EncryptionService;
import fitconnect.api.domain.metadata.Metadata;
import fitconnect.api.domain.metadata.attachment.Attachment;
import fitconnect.api.domain.metadata.data.Data;
import fitconnect.api.metadata.MetadataService;
import fitconnect.api.problems.EncryptionProblem;
import fitconnect.api.validation.CertificateValidator;
import fitconnect.api.validation.ValidationResult;

import java.util.List;
import java.util.Optional;

/**
 * A technical system that creates a submission via the FIT-Co Submission API.
 */
public class SubmissionSender implements OAuthService, EncryptionService, MetadataService, CertificateValidator {

    private final OAuthService authService;
    private final EncryptionService encryptionService;

    public SubmissionSender(final OAuthService authService, final EncryptionService encryptionService){
        this.authService = authService;
        this.encryptionService = encryptionService;
    }

    @Override
    public Optional<OAuthToken> authenticate(String clientId, String clientSecret, String... scope) {
        return authService.authenticate(clientId, clientSecret, scope);
    }

    @Override
    public String encryptString(RSAKey publicKey, String unencryptedData) throws EncryptionProblem {
        return encryptionService.encryptString(publicKey, unencryptedData);
    }

    @Override
    public String encryptBytes(RSAKey publicKey, byte[] unencryptedAttachment) throws EncryptionProblem {
        return encryptionService.encryptBytes(publicKey, unencryptedAttachment);
    }

    @Override
    public Metadata createMetadata(Data data, List<Attachment> attachments, RSAKey publicKey) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validatePublicKey(final RSAKey publicKey) {
        throw new UnsupportedOperationException("not yet implemented");
    }

}
