package fitconnect.impl;

import com.nimbusds.jose.jwk.RSAKey;
import fitconnect.api.auth.OAuthService;
import fitconnect.api.auth.OAuthToken;
import fitconnect.api.crypto.DecryptionService;
import fitconnect.api.domain.metadata.Metadata;
import fitconnect.api.problems.DecryptionProblem;
import fitconnect.api.validation.MetadataValidator;
import fitconnect.api.validation.ValidationResult;

import java.util.Optional;

/**
 * The technical system that accepts submissions on the administration side.
 */
public class SubmissionSubscriber implements OAuthService, DecryptionService, MetadataValidator {

    private final OAuthService authService;
    private final DecryptionService decryptionService;

    public SubmissionSubscriber(final OAuthService authService, final DecryptionService decryptionService){
        this.authService = authService;
        this.decryptionService = decryptionService;
    }

    @Override
    public Optional<OAuthToken> authenticate(String clientId, String clientSecret, String... scope) {
        return authService.authenticate(clientId, clientSecret, scope);
    }

    @Override
    public String decryptString(RSAKey privateKey, String encryptedData) throws DecryptionProblem {
        return decryptionService.decryptString(privateKey, encryptedData);
    }

    @Override
    public byte[] decryptBytes(RSAKey privateKey, String encryptedData) throws DecryptionProblem {
        return decryptionService.decryptBytes(privateKey, encryptedData);

    }

    @Override
    public ValidationResult validateMetadataSchema(Metadata metadata, String jsonSchema) {
        throw new UnsupportedOperationException("not yet implemented");
    }

    @Override
    public ValidationResult validateMetadataHashValues(Metadata metadata) {
        throw new UnsupportedOperationException("not yet implemented");
    }
}
