package fitconnect.impl.crypto;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.Arrays;

public class MetadataVerifier {

    private static final String DEFAULT_ALGORITHM = "SHA-512"; // Currently, only SHA-512 is supported.

    private final MessageDigest hasher;

    public MetadataVerifier() {
        try {
            this.hasher = MessageDigest.getInstance(DEFAULT_ALGORITHM);
        } catch (NoSuchAlgorithmException e) {
            //TODO use Problem Exception Type
            throw new RuntimeException(e);
        }
    }

    public byte[] createHash(byte[] data) {
        this.hasher.reset();
        return this.hasher.digest(data);
    }

    public boolean verify(byte[] originalHash, byte[] data) {
        byte[] newHash = createHash(data);
        return compareHashes(originalHash, newHash);
    }

    public static String toHexString(byte[] hash) {
        return String.format("%0128x", new BigInteger(1, hash));
    }

    public byte[] fromHexString(String hexString) {
        byte[] byteArray = new BigInteger(hexString, 16).toByteArray();
        return Arrays.copyOfRange(byteArray, 1, byteArray.length);
    }

    private boolean compareHashes(byte[] originalHash, byte[] comparisonHash) {
        int diff = originalHash.length ^ comparisonHash.length;
        for (int i = 0; i < originalHash.length && i < comparisonHash.length; i++) {
            diff |= originalHash[i] ^ comparisonHash[i];
        }
        return diff == 0;
    }
}
